#include <iostream>
#include <vector>
using namespace std;


//ROOT libraries
#include "TTree.h"
#include "TFile.h"
#include "TLorentzVector.h"
#include "TVector3.h"
#include "TH1.h"
#include "TH2.h"
#include "TF1.h"

//My include file          
#include "myTree.h"

struct myBOI_t {
  TLorentzVector mo;
  int pid;
  int sciDet;
  int calLayer;
  TVector3 vert;
  double calLu;
  double calLv;
  double calLw;
};
//function prototype
void printUsage();

int main(int argc, char **argv) {
  cout<<"Hello World"<<endl;
  char inFileName[120];
  char outFileName[120];
  //Define integer nToRun;
  int nToRun = 1;
  char *argptr;
  sprintf(outFileName,"./myOutFile.root");
  if (argc == 1) printUsage();
  for (int i=1; i<argc; i++) {
    argptr = argv[i];
    if (*argptr == '-') {
      argptr++;
      switch (*argptr) {
      case 'h':
        printUsage();
        break;
      case 'n':
        nToRun = atoi(++argptr);
        break;
      case 'o':
        strcpy(outFileName,++argptr);
        break;
      default:
        printUsage();
        break;
      }
    } else {
      strcpy(inFileName,argptr);
    }
  }
  //Define the histograms                                               

  TH1D *hmassPro   = new TH1D("hmassPro","",120,0.8,1.2);
  TH1D *hmassPip   = new TH1D("hmassPip","",120,0.8,1.2);
  TH1D *hmassPim   = new TH1D("hmassPim","",120,0.8,1.2);
  TH1D *hmassKp   = new TH1D("hmassKp","",120,0.8,1.2);
  TH1D *hmassKm   = new TH1D("hmassKm","",120,0.8,1.2);

  
  TH2D *hBetaVsP     = new TH2D("hBetaVsP","",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPPro  = new TH2D("hBetaVsPPro","",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPPip  = new TH2D("hBetaVsPPip","",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPPim  = new TH2D("hBetaVsPPim","",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPKp  = new TH2D("hBetaVsPKp","",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPKm  = new TH2D("hBetaVsPKm","",60,0.0,6.0,110,0.0,1.1);

  TH1D *hMassKpKm = new TH1D("hMassKpKm","",100,0.9,1.5);
  
  //Setup the output file                                                   
  TFile *outFile = new TFile(outFileName,"RECREATE");

  //Setup the input file and tree                                          
  TFile* inFile = new TFile(inFileName);
  
  //Get tree out of the file
  TTree *inTree = (TTree*)inFile->Get("clas12");
  
  //Define the tree structure
  myTree_t myTree;
  
  //Get the tree branches into myTree structure
  getBranchesStage1(inTree,&myTree);
  
  //Find out how many events are in the tree and set to nToRun
  nToRun = inTree->GetEntries();
  
  //Print the number of events to process in the terminal
  cout<<"Number of events to process = "<<nToRun<<endl;
  
  
  for(Int_t i=0;i<nToRun;i++){
    
    inTree->GetEntry(i);
    //Print every 50000 events                                          
    if (i%50000 == 0) cout<<"Events processed = "<<i<<endl;
    //Find out how many particles are in the event                      
    int nREC_Particle = myTree.REC_Particle_pid->size();
    int nRECFT_Particle = myTree.RECFT_Particle_pid->size();
    int nREC_Calorimeter = myTree.REC_Calorimeter_pindex->size();
    
    //Setting up Bucket of Information 
    myBOI_t boi;
    vector <myBOI_t> proBOI;
    vector <myBOI_t> elecBOI;
    vector <myBOI_t> pipBOI;
    vector <myBOI_t> pimBOI;
    vector <myBOI_t> kpBOI;
    vector <myBOI_t> kmBOI;
    vector <myBOI_t> betaBOI;
    vector <myBOI_t> phoBOI;
    vector <myBOI_t> phoAllBOI;

    //Loop over each particle in the event                               
    for(int index = 0; index<nRECFT_Particle; index++){

      int pid   = myTree.RECFT_Particle_pid->at(index);
      int partStat = myTree.RECFT_Particle_status->at(index);
      int partStatDet = partStat - partStat%1000;
      boi.sciDet = partStatDet;
      double vx = myTree.REC_Particle_vx->at(index);
      double vy = myTree.REC_Particle_vy->at(index);
      double vz = myTree.REC_Particle_vz->at(index);
      boi.vert.SetXYZ(vx,vy,vz);
      double px = myTree.REC_Particle_px->at(index);
      double py = myTree.REC_Particle_py->at(index);
      double pz = myTree.REC_Particle_pz->at(index);
      double p  = sqrt(pow(px,2)+pow(py,2)+pow(pz,2));
      double beta   = myTree.RECFT_Particle_beta->at(index);
      double gamma = -1;
      

      if (beta != 1)  gamma = sqrt(1/(1-beta*beta));
      double massSQ = pow(p/(gamma*beta),2);
      double mass = -1;
      double eval = -1;
      
      
      if (massSQ>0) mass = sqrt(massSQ);

      hBetaVsP->Fill(p,beta);      
      
      //eft -> electron seen in forward tagger
      if(pid == 11){
	double eft = sqrt(pow(p,2)+pow(0.000511,2));
     	boi.mo.SetPxPyPzE(px,py,pz,p);
	double eftNew = eft-0.03689+0.1412*eft-0.04316*pow(eft,2)+0.007046*pow(eft,3)-0.0004055*pow(eft,4);
	double eftFrac = eftNew/eft;
	eftFrac = 1.0888 - 0.0189 * eft + 0.0031 * pow(eft,2)- 0.0002 * pow(eft,3);
	TLorentzVector p4VecCor(px*eftFrac,py*eftFrac,pz*eftFrac,p*eftFrac);
	boi.mo = p4VecCor;
	elecBOI.push_back(boi);
      }

      hBetaVsP->Fill(p,beta);

      //Proton stuff
      if (pid == 2212 && mass>0) {
	hmassPro->Fill(mass);
	hBetaVsPPro->Fill(p,beta);
	eval = sqrt(pow(p,2)+pow(0.938,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	proBOI.push_back(boi);
      }
      //pi+ stuff
      if (pid == 211) {
	hmassPip->Fill(mass); 
	hBetaVsPPip->Fill(p,beta);
	eval = sqrt(pow(p,2)+pow(0.139570,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	pipBOI.push_back(boi);
      }
      //pi+ stuff
      if (pid == -211) {
	hmassPim->Fill(mass); 
	hBetaVsPPim->Fill(p,beta);
	eval = sqrt(pow(p,2)+pow(0.139570,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	pimBOI.push_back(boi);
      }
      //K+ stuff - Beta cut beta>0.855
      if (pid == 321 && mass>0 && beta>0.855) {
	hmassKp->Fill(mass);
	hBetaVsPKp->Fill(p,beta); 
	eval = sqrt(pow(p,2)+pow(0.493677,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	if (partStatDet == 2000) kpBOI.push_back(boi);
      }
      //K- stuff - Beta cut beta>0.855
      if (pid == -321 && mass>0 && beta>0.855) {
	hmassKm->Fill(mass); 
	hBetaVsPKm->Fill(p,beta);
	eval = sqrt(pow(p,2)+pow(0.493677,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	if (partStatDet == 2000) kmBOI.push_back(boi);
      }
      
      //Photon Stuff
      if(pid == 22){
	//cout<<"-----------nCal ="<<nREC_Calorimeter<<endl;                    
	int nphoAll = phoAllBOI.size();
	boi.mo.SetPxPyPzE(px,py,pz,p);
	if (p > 0.15) phoAllBOI.push_back(boi);
	if (p < 0.15) continue;
	if (nphoAll == 2) continue;
	for(int calIndex = 0; calIndex<nREC_Calorimeter; calIndex++){
	  if (index == myTree.REC_Calorimeter_pindex->at(calIndex)){
	    boi.calLayer = myTree.REC_Calorimeter_layer->at(calIndex);
	    boi.calLu = myTree.REC_Calorimeter_lu->at(calIndex);
	    boi.calLv = myTree.REC_Calorimeter_lv->at(calIndex);
	    boi.calLw = myTree.REC_Calorimeter_lw->at(calIndex);
	    if (boi.calLayer == 1 && boi.calLv > 19 && boi.calLw > 19) {
	      phoBOI.push_back(boi);                                                
	    }
	  }
	}
      }
    }

    //REACTION CODE HERE

    int npro = proBOI.size();
    int nkp = kpBOI.size();
    int nelec = elecBOI.size();
    int nkm = kmBOI.size();
    int npho = phoBOI.size();
    int nphoAll = phoAllBOI.size();


    if (nkp == 1 && nkm == 1){
      TLorentzVector moKpKm = kpBOI[0].mo + kmBOI[0].mo;
      hMassKpKm->Fill(moKpKm.Mag());
    }
  }






  outFile->cd();

  hBetaVsP->Write();
  hBetaVsPPro->Write();
  hMassKpKm->Write();
  
  outFile->Write();


  return 0;
}
void printUsage(){
  fprintf(stderr,"\nUsage:");
  fprintf(stderr,"\nstage2 [-switches] <inputFile>\n");
  fprintf(stderr,"\nSWITCHES:\n");
  fprintf(stderr,"-n\tNumber of events to process. Default behavior is to run all events\n");
  fprintf(stderr,"-h\tPrint this message\n");
  fprintf(stderr,"-o<arg>\tOutFileName. Default is myOutFile.root\n\n");

  cout<<"The current default operation is equivalent to the command:"<<endl;
  cout<<"myTreeCodeV3 -omyOutFile.root inputFile ,"<<endl;
  cout<<"where \"inputFile\" must be supplied by the user\n"<<endl;

  exit(0);
}





